RMS.LoadLibrary("rmgen");

const BUILDING_ANGlE = -PI/4;

// initialize map

log("Initializing map...");

InitMap();

var numPlayers = getNumPlayers();
var mapSize = getMapSize();

// create tile classes

// var clPlayer = createTileClass();
// var clPath = createTileClass();
// var clHill = createTileClass();
// var clForest = createTileClass();
// var clWater = createTileClass();
// var clRock = createTileClass();
// var clFood = createTileClass();
// var clBaseResource = createTileClass();

// // randomize player order
// var playerIDs = [];
// for (var i = 0; i < numPlayers; i++)
// {
	// playerIDs.push(i+1);
// }
// playerIDs = sortPlayers(playerIDs);

// // place players

// var playerX = new Array(numPlayers);
// var playerZ = new Array(numPlayers);
// var playerAngle = new Array(numPlayers);

// var startAngle = randFloat() * 2 * PI;
// for (var i=0; i < numPlayers; i++)
// {
	// playerAngle[i] = startAngle + i*2*PI/numPlayers;
	// playerX[i] = 0.5 + 0.39*cos(playerAngle[i]);
	// playerZ[i] = 0.5 + 0.39*sin(playerAngle[i]);
// }

// for (var i=0; i < numPlayers; i++)
// {
	// var id = playerIDs[i];
	// log("Creating base for player " + id + "...");
	
	// // get the x and z in tiles
	// var fx = fractionToTiles(playerX[i]);
	// var fz = fractionToTiles(playerZ[i]);
	// var ix = round(fx);
	// var iz = round(fz);

	// create starting units
	// placeCivDefaultEntities(fx, fz, id, BUILDING_ANGlE);
// }


//////////
// Test discrete vector analysis functionality
//////////
// var testScalarField = getScalarField(2, 5); // Get an "empty" scalar field (all values = 0)
// var testCoordinates = getAllCoordinatesInHypercube(2, 5); // Get all valid (integer) coordinates of the area the field is defined in
// for (var i = 0; i < testCoordinates.length; i++)
	// setFieldValueByCoordinate(testScalarField, testCoordinates[i], randInt(-9, 9)); // Ranomize the scalar fields values
// var gradField = getGrad(testScalarField); // Get the gradient field (a vector field) determining the slope in each coordinated direction (or the direction water would flow)
// var divGradField = getDiv(gradField); // Get the divergence of the gradient field (Would be something like how much water will leave this coordinate more then gathered from other coordinates)
// var normalizedTestScalarField = getNormalizedField(testScalarField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// var normalizedGradField = getNormalizedField(gradField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// var normalizedDivGradField = getNormalizedField(divGradField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// log("uneval(testScalarField) = " + uneval(testScalarField));
// log("uneval(normalizedTestScalarField) = " + uneval(normalizedTestScalarField));
// log("uneval(gradField) = " + uneval(gradField));
// log("uneval(normalizedGradField) = " + uneval(normalizedGradField));
// log("uneval(divGradField) = " + uneval(divGradField));
// log("uneval(normalizedDivGradField) = " + uneval(normalizedDivGradField));


//////////
// Heightmap functionality
//////////

// Some heightmap variables
const MIN_HEIGHT = - SEA_LEVEL; // 20
const MAX_HEIGHT = 0xFFFF/HEIGHT_UNITS_PER_METRE - SEA_LEVEL; // A bit smaler than 90

// Random heightmap generation
function getRandomReliefmap(minHeight, maxHeight)
{
	minHeight = (minHeight || MIN_HEIGHT);
	maxHeight = (maxHeight || MAX_HEIGHT);
	if (minHeight < MIN_HEIGHT)
		warn("getRandomReliefmap: Argument minHeight is smaler then the supported minimum height of " + MIN_HEIGHT + " (const MIN_HEIGHT): " + minHeight)
	if (maxHeight > MAX_HEIGHT)
		warn("getRandomReliefmap: Argument maxHeight is smaler then the supported maximum height of " + MAX_HEIGHT + " (const MAX_HEIGHT): " + maxHeight)
	var reliefmap = [];
	for (var x = 0; x <= mapSize; x++)
	{
		reliefmap.push([]);
		for (var y = 0; y <= mapSize; y++)
		{
			reliefmap[x].push(randFloat(minHeight, maxHeight));
		}
	}
	return reliefmap;
}

// Apply a heightmap
function setReliefmap(reliefmap)
{
	// g_Map.height = reliefmap;
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			setHeight(x, y, reliefmap[x][y]);
		}
	}
}

// Get the diferrence between minimum and maxumum height
function getMinAndMaxHeight(reliefmap)
{
	var height = {};
	height.min = Infinity;
	height.max = -Infinity;
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			if (reliefmap[x][y] < height.min)
				height.min = reliefmap[x][y];
			else if (reliefmap[x][y] > height.max)
				height.max = reliefmap[x][y];
		}
	}
	return height;
}

// Normalizing the heightmap
function getRescaledReliefmap(reliefmap, minHeight, maxHeight)
{
	var newReliefmap = deepcopy(reliefmap);
	minHeight = (minHeight || MIN_HEIGHT);
	maxHeight = (maxHeight || MAX_HEIGHT);
	if (minHeight < MIN_HEIGHT)
		warn("getRescaledReliefmap: Argument minHeight is smaler then the supported minimum height of " + MIN_HEIGHT + " (const MIN_HEIGHT): " + minHeight)
	if (maxHeight > MAX_HEIGHT)
		warn("getRescaledReliefmap: Argument maxHeight is smaler then the supported maximum height of " + MAX_HEIGHT + " (const MAX_HEIGHT): " + maxHeight)
	var oldHeightRange = getMinAndMaxHeight(reliefmap);
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			newReliefmap[x][y] = minHeight + (reliefmap[x][y] - oldHeightRange.min) / (oldHeightRange.max - oldHeightRange.min) * (maxHeight - minHeight);
		}
	}
	return newReliefmap
}

function drawFieldAbsOnTiles(field, isVectorField)
{
	if (isVectorField === undefined)
		isVectorField = (typeof(field[0][0]) == "object");
	var fieldMaxValue = getFieldsMaxAbsValue2d(field, isVectorField);
	for (var x = 0; x < mapSize; x++)
	{
		for (var y = 0; y < mapSize; y++)
		{
			if (isVectorField)
				var fieldActualValue = Math.pow(field[x][y].x * field[x][y].x + field[x][y].y * field[x][y].y, 1/2);
			else
				var fieldActualValue = abs(field[x][y]);
			if (fieldActualValue < 1/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "yellow");
			else if (fieldActualValue < 2/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "neon green");
			else if (fieldActualValue < 3/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "green");
			else if (fieldActualValue < 4/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "light blue");
			else if (fieldActualValue < 5/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "blue");
			else if (fieldActualValue < 6/7 * fieldMaxValue)
				g_Map.setTexture(x, y, "purple");
			else if (fieldActualValue != 0)
				g_Map.setTexture(x, y, "red");
		}
	}
}

/*
Apply named erosion functions
*/

// Applying decay erosion (terrain independent)
function getHeightErrosionedReliefmap(reliefmap, strength)
{
	var newReliefmap = deepcopy(reliefmap);
	strength = (strength || 1.0); // Values much higher then 1 (1.32+ for an 8 tile map, 1.45+ for a 12 tile map, 1.62+ @ 20 tile map, 0.99 @ 4 tiles) will result in a resonance disaster/self interference
	// var map = [[1, 0], [0, 1], [-1, 0], [0, -1]];
	var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1]]; // Default
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [0, 2], [-2, 0], [0, -2]];
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [2, 1], [1, 2], [0, 2], [-1, 2], [-2, 1], [-2, 0], [-2, -1], [-1, -2], [0, -2], [1, -2], [2, -1]];
	// var map = [[1, 0], [1, 1], [0, 1], [0, -1], [1, -1]]; // Directed, only effected from right
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			var div = 0;
			for (var i = 0; i < map.length; i++)
				newReliefmap[x][y] += strength / map.length * (reliefmap[(x + map[i][0] + mapSize + 1) % (mapSize + 1)][(y + map[i][1] + mapSize + 1) % (mapSize + 1)] - reliefmap[x][y]); // Not entirely sure if scaling with map.length is perfect but tested values seam to indicate it is
		}
	}
	return newReliefmap;
}

// Applying additive directed erosion (terrain independent, carries material left) NOTE: The average height will grow so it has to be rescaled after applying this!
// Results in round high ground structures
function getAdditiveWindErrosionedReliefmap(reliefmap, strength)
{
	var newReliefmap = deepcopy(reliefmap);
	strength = (strength || 1.0); // Values much higher then 1 will result in a resonance disaster/self interference
	// var map = [[1, 0], [0, 1], [-1, 0], [0, -1]];
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1]]; // Default
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [0, 2], [-2, 0], [0, -2]];
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [2, 1], [1, 2], [0, 2], [-1, 2], [-2, 1], [-2, 0], [-2, -1], [-1, -2], [0, -2], [1, -2], [2, -1]];
	var map = [[1, 0], [1, 1], [0, 1], [0, -1], [1, -1]]; // Directed, only effected from right
	// var map = [[0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1]]; // Directed, only effected from left
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			for (var i = 0; i < map.length; i++)
			{
				var add = strength / map.length * (reliefmap[(x + map[i][0] + mapSize + 1) % (mapSize + 1)][(y + map[i][1] + mapSize + 1) % (mapSize + 1)] - reliefmap[x][y]);
				if (add > 0)
					newReliefmap[x][y] += add;
			}
		}
	}
	return newReliefmap;
}

// Applying substractive directed erosion (terrain independent, carries material left) NOTE: The average height will shrink so it has to be rescaled after applying this!
// Results in round low ground structures
function getSubstractiveWindErrosionedReliefmap(reliefmap, strength)
{
	var newReliefmap = deepcopy(reliefmap);
	strength = (strength || 1.0); // Values much higher then 1 will result in a resonance disaster/self interference
	// var map = [[1, 0], [0, 1], [-1, 0], [0, -1]];
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1]]; // Default
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [0, 2], [-2, 0], [0, -2]];
	// var map = [[1, 0], [1, 1], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [2, 0], [2, 1], [1, 2], [0, 2], [-1, 2], [-2, 1], [-2, 0], [-2, -1], [-1, -2], [0, -2], [1, -2], [2, -1]];
	// var map = [[1, 0], [1, 1], [0, 1], [0, -1], [1, -1]]; // Directed, only effected from right
	var map = [[0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1]]; // Directed, only effected from left
	for (var x = 0; x <= mapSize; x++)
	{
		for (var y = 0; y <= mapSize; y++)
		{
			for (var i = 0; i < map.length; i++)
			{
				var add = strength / map.length * (reliefmap[(x + map[i][0] + mapSize + 1) % (mapSize + 1)][(y + map[i][1] + mapSize + 1) % (mapSize + 1)] - reliefmap[x][y]);
				if (add < 0)
					newReliefmap[x][y] += add;
			}
		}
	}
	return newReliefmap;
}


//////////
// Heightmap functionality based on discrete vector analysis
//////////

function getErosionHeightmapDVA(heightmap, strength)
{
	strength = (strength || 1);
	var size = heightmap.length;
	var newHeightmap = deepcopy(heightmap);
	var divGradMap = getGrad(newHeightmap);
	divGradMap = getDiv(divGradMap);
	for (var x = 0; x < size; x++)
	{
		for (var y = 0; y < size; y++)
		{
			newHeightmap[x][y] -= strength / 4/* 2 * dimension*/  * divGradMap[x][y];
		}
	}
	return newHeightmap;
}
// Test
// var dimension = 2;
// var size = 3
// var testScalarField = getScalarField(dimension, size); // Get an "empty" scalar field (all values = 0)
// var testCoordinates = getAllCoordinatesInHypercube(dimension, size); // Get all valid (integer) coordinates of the area the field is defined in
// for (var i = 0; i < testCoordinates.length; i++)
	// setFieldValueByCoordinate(testScalarField, testCoordinates[i], randInt(-9, 9)); // Ranomize the scalar fields values
// var gradField = getGrad(testScalarField); // Get the gradient field (a vector field) determining the slope in each coordinated direction (or the direction water would flow)
// var divGradField = getDiv(gradField); // Get the divergence of the gradient field (Would be something like how much water will leave this coordinate more then gathered from other coordinates)
// var normalizedTestScalarField = getNormalizedField(testScalarField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// var normalizedGradField = getNormalizedField(gradField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// var normalizedDivGradField = getNormalizedField(divGradField); // Normalized version, needed for non-linear effects e.g. roughtly considering friction
// log("uneval(testScalarField) = " + uneval(testScalarField));
// log("uneval(normalizedTestScalarField) = " + uneval(normalizedTestScalarField));
// log("uneval(gradField) = " + uneval(gradField));
// // log("uneval(normalizedGradField) = " + uneval(normalizedGradField));
// log("uneval(divGradField) = " + uneval(divGradField));
// // log("uneval(normalizedDivGradField) = " + uneval(normalizedDivGradField));
// var erosionedTestScalarField = getErosionHeightmapDVA(testScalarField);
// log("uneval(erosionedTestScalarField) = " + uneval(erosionedTestScalarField));
// var normalizedErosionedField = getNormalizedField(erosionedTestScalarField);
// log("uneval(normalizedErosionedField) = " + uneval(normalizedErosionedField));

function getFloatingWaterErosionHeightmapDVA(heightmap, strength)
{
	strength = (strength || 1);
	var size = heightmap.length;
	var newHeightmap = deepcopy(heightmap);
	var divGradMap = getGrad(newHeightmap);
	divGradMap = getDiv(divGradMap);
	var maxValue = getFieldsMaxAbsValue(divGradMap, false);
	divGradMap = getNormalizedField(divGradMap, 1, false);
	for (var x = 0; x < size; x++)
	{
		for (var y = 0; y < size; y++)
			newHeightmap[x][y] -= strength / 4/* 2 * dimension */  * maxValue * Math.pow(divGradMap[x][y], 2);
	}
	return newHeightmap;
}

/*
...2D
*/

/*
getDecayErosionedHeightmapDVA2d

Arguments:
	heightmap:	An array of x arrays of y floats. A discrete scalar field with "width" x and "height" y (usually a hightmap).
	strength:	A float value between 0 and 1. Higher values than 1 may cause self interference when applied multiple times.
*/
function getDecayErosionedHeightmapDVA2d(heightmap, strength, wrap)
{
	strength = (strength || 0.8); // For some reason 1 is to high causing resonance.
	if (wrap === undefined)
		wrap = false;
	var newHeightmap = deepcopy(heightmap);
	var divGrad = getDiv2d(getGrad2d(heightmap, wrap), wrap);
	for (var x = 0; x < heightmap.length; x++)
	{
		for (var y = 0; y < heightmap[x].length; y++)
			newHeightmap[x][y] = heightmap[x][y] - strength * divGrad[x][y];
	}
	return newHeightmap;
}

function getWindErosionedHeightmapDVA2d(heightmap, windVektor, strength, wrap)
{
	windVektor = (windVektor || {"x": 0, "y": 0});
	strength = (strength || 1);
	if (wrap === undefined)
		wrap = true;
	var newHMap = deepcopy(heightmap);
	var gradHMap = getGrad2d(heightmap, wrap);
	for (var x = 0; x < gradHMap.length; x++)
	{
		for (var y = 0; y < gradHMap[x].length; y++)
		{
			gradHMap[x][y].x += 1000 * windVektor.x;
			gradHMap[x][y].y += 1000 * windVektor.y;
		}
	}
	var divGradHMap = getDiv2d(gradHMap, wrap);
	for (var x = 0; x < heightmap.length; x++)
	{
		for (var y = 0; y < heightmap[x].length; y++)
			newHMap[x][y] = heightmap[x][y] + strength * divGradHMap[x][y] / 5; // Divisor has to be higher than 4.1 otherwise self interference/resonance desaster, not sure why this value
	}
	return newHMap;
}

// Not working as planned...
function getFloatingWaterErosionedHeightmapDVA2d(heightmap, strength)
{
	strength = (strength || 1);
	var newHeightmap = deepcopy(heightmap);
	var floatField = getGrad2d(heightmap);
	var maxValue = 0;
	for (var x = 0; x < floatField.length; x++)
	{
		for (var y = 0; y < floatField[x].length; y++)
		{
			var length = Math.pow(floatField[x][y].x * floatField[x][y].x + floatField[x][y].y * floatField[x][y].y, 1/2);
			if (length > maxValue)
				maxValue = length;
		}
	}
	var nonLinearGrad = deepcopy(floatField);
	for (var x = 0; x < floatField.length; x++)
	{
		for (var y = 0; y < floatField[x].length; y++)
		{
			// nonLinearGrad[x][y].x = floatField[x][y].x * abs(floatField[x][y].x) / maxValue;
			// nonLinearGrad[x][y].y = floatField[x][y].y * abs(floatField[x][y].y) / maxValue;
			nonLinearGrad[x][y].x = Math.pow(abs(floatField[x][y].x), 1/2) * floatField[x][y].x / abs(floatField[x][y].x) / maxValue;
			nonLinearGrad[x][y].y = Math.pow(abs(floatField[x][y].y), 1/2) * floatField[x][y].y / abs(floatField[x][y].y) / maxValue;
		}
	}
	var deltaHeightmap = getDiv2d(nonLinearGrad);
	for (var x = 0; x < heightmap.length; x++)
	{
		for (var y = 0; y < heightmap[x].length; y++)
			newHeightmap[x][y] = heightmap[x][y] + strength * deltaHeightmap[x][y] / 100; // CHECK DIVISOR!!!
	}
	return newHeightmap;
}

/*
Visualize water drain to get the water erosion right
*/
function paintTilesWithWaterDrain(heightmap, addedWater, accuracy, wrap)
{
	addedWater = (addedWater || 1);
	wrap = (wrap || false);
	accuracy = (accuracy || 1/2);
	var waterHightMap = getRandomReliefmap(addedWater, addedWater);
	
	// Calculate water drain loop
	var minWaterHight = addedWater; // Break condition
	var waterDrainTotalField = getGrad2d(getRandomReliefmap(0, 0), wrap);
	var loop = 0;
	log("Entering while loop...");
	while (minWaterHight > accuracy)
	{
		loop++;
		if (loop % 100 == 0)
			log("minWaterHight = " + minWaterHight + " after " + loop + " loops...");
		
		// Calculate waterSurfaceMap
		var waterSurfaceMap = heightmap;
		for (var x = 0; x < waterSurfaceMap.length; x++)
		{
			for (var y = 0; y < waterSurfaceMap[x].length; y++)
			{
				waterSurfaceMap[x][y] += waterHightMap[x][y];
			}
		}
		
		// Calculate theoretical water drain
		var waterDrainField = getGrad2d(waterSurfaceMap, wrap);
		
		// Add waterDrainField to waterDrainTotalField
		for (var x = 0; x < waterDrainTotalField.length; x++)
		{
			for (var y = 0; y < waterDrainTotalField[x].length; y++)
			{
				waterDrainTotalField[x][y].x += waterDrainField[x][y].x;
				waterDrainTotalField[x][y].y += waterDrainField[x][y].y;
			}
		}
		
		// Calculate and rescale Scale waterDrainMap to fit accuracy
		var waterDrainMap = getDiv2d(waterDrainField, wrap);
		waterDrainMap = getRescaledField2D(waterDrainMap, accuracy);
		
		// Recalculate waterHightMap for break condition and next loop
		for (var x = 0; x < waterHightMap.length; x++)
		{
			for (var y = 0; y < waterHightMap[x].length; y++)
			{
				waterHightMap[x][y] -= 0.8 * min(waterDrainMap[x][y], waterHightMap[x][y]);
				if (waterHightMap[x][y] < 0)
					warn("Negative water hight!");
			}
		}
		
		// Recalculate minWaterHight for break condition
		minWaterHight = addedWater;
		for (var x = 0; x < waterHightMap.length; x++)
		{
			for (var y = 0; y < waterHightMap[x].length; y++)
			{
				if (waterHightMap[x][y] < minWaterHight)
					minWaterHight = waterHightMap[x][y];
			}
		}
		
	}
	log("Number of loops needed: " + loop);
	
	// Paint tiles
	drawFieldAbsOnTiles(waterDrainTotalField);
}

/*
Try rain driven erosion another time...
Has to be changed to be more tidy with added DVA(2D) functions (e.g. getRandomScalarField2D, getRandomVectorField2D, getSumOfFields2D, ...)
(ATM functions from this map are mixed with DVA(2D) functions - which is bad!)
Is it really needed to scale things to water plane (slow!) or could the water just be "small" compared to the hight and only take into account the hight?
NOTE: This got some resonance! Check all used functions und extend library - then continue...
*/
function getRainErosionedHeightmapDVA2D(heightmap, rainfall, cycles, strength, wraped)
{
	rainfall = (rainfall || 1); // Should correspond to water level/average water coverage! How?
	cycles = (cycles || 10);
	strength = (strength || 1);
	wraped = (wraped || false);
	var waterMap = getRandomReliefmap(rainfall, rainfall); // Randomize? Changed each cycle!
	var waterDrainFieldTotal = getGrad2d(waterMap, wraped); // Wrapped OK? If changed erosionFloatField needs to be changed as well!
	// Water drain simulation cycles
	var waterMapMin = rainfall;
	var waterMapMinLast = rainfall;
	var loop = 0
	while (waterMapMin > 1 / cycles)
	// for (var i = 0; i < cycles; i++)
	{
		loop++;
		if (loop % 1000 == 0)
			log("loop = " + loop);
		// Calculate water surface height
		var waterSurfaceMap = heightmap;
		for (var x = 0; x < waterSurfaceMap.length; x++)
		{
			for (var y = 0; y < waterSurfaceMap[x].length; y++)
			{
				waterSurfaceMap[x][y] += waterMap[x][y];
			}
		}
		// Calculate water drain and rescale it to avoid resonance
		var waterDrainField = getGrad2d(waterSurfaceMap, wraped); // Wrapped OK? If changed erosionFloatField needs to be changed as well!
		var maxSteppness = getFieldsMaxAbsValue2d(waterDrainField); // To avoid resonance, sane? OK to ignore water hight changes (so calculate only once befor loop)?
		waterDrainField = getRescaledField2D(waterDrainField, rainfall / cycles); // Better to rescale the waterHightChangeMap!!!
		// Add water drain to waterDrainFieldTotal
		for (var x = 0; x < waterDrainField.length; x++)
		{
			for (var y = 0; y < waterDrainField[x].length; y++)
			{
				waterDrainFieldTotal[x][y].x += waterDrainField[x][y].x;
				waterDrainFieldTotal[x][y].y += waterDrainField[x][y].y;
			}
		}
		// Apply calculated water drain to the water map for next loop and set break condition
		var waterHightChangeMap = getDiv2d(waterDrainField, wraped);
		waterMapMinLast = deepcopy(waterMapMin);
		waterMapMin = rainfall;
		for (var x = 0; x < waterMap.length; x++)
		{
			for (var y = 0; y < waterMap[x].length; y++)
			{
				waterMap[x][y] += waterHightChangeMap[x][y];
				if (waterMap[x][y] < waterMapMin)
					waterMapMin = waterMap[x][y];
				// DEBUG Start
				if (waterMap[x][y] < 0)
					warn("waterMap include negative values!");
				// DEBUG End
			}
		}
		if (loop % 1000 == 0)
			log("waterMapMin = " + waterMapMin);
		if (waterMapMin > waterMapMinLast)
			warn("waterMapMin raised from " + waterMapMinLast + " to " + waterMapMin);
	}
	// DEBUG Start: Check waterMap
	var waterMapMin = 1000000;
	var waterMapMax = -1000000;
	for (var x = 0; x < waterMap.length; x++)
	{
		for (var y = 0; y < waterMap[x].length; y++)
		{
			if (waterMap[x][y] < waterMapMin)
				waterMapMin = waterMap[x][y];
			if (waterMap[x][y] > waterMapMax)
				waterMapMax = waterMap[x][y];
		}
	}
	log("waterMapMin = " + waterMapMin);
	log("waterMapMax = " + waterMapMax);
	// DEBUG End: Check waterMap
	// Apply errosion to hightmap
	var newHeightmap = deepcopy(heightmap);
	var hightChangeMap = getDiv2d(waterDrainFieldTotal, wraped);
	// var originalSteppness = getFieldsMaxAbsValue2d(heightmap);
	// var maxHightChange = getFieldsMaxAbsValue2d(hightChangeMap);
	// hightChangeMap = getRescaledField2D(hightChangeMap, strength); // Not very sane! Rethink if dependent on water hight/coverage!
	for (var x = 0; x < newHeightmap.length; x++)
	{
		for (var y = 0; y < newHeightmap[x].length; y++)
		{
			newHeightmap[x][y] -= hightChangeMap[x][y]; // WHY - ???
		}
	}
	
	return newHeightmap;
}

function getWaterDrainMap(heightmap, seepage, waterMap)
{
	// Setting optional parameters
	seepage = (seepage || 0)
	if (!waterMap)
	{
		waterMap = deepcopy(heightmap);
		for (var x = 0; x < heightmap.length; x++)
			for (var y = 0; y < heightmap[x].length; y++)
					waterMap[x][y] = 1;
	}
	
	// Reorganize tiles in list and sort by height
	var tilesByHeight = [];
	for (var x = 0; x < heightmap.length; x++)
		for (var y = 0; y < heightmap[x].length; y++)
			tilesByHeight.push({"height" : heightmap[x][y], "x" : x, "y" : y});
	
	function sortByHeight(heightCoordinatePair1, heightCoordinatePair2)
	{
		return heightCoordinatePair1.height - heightCoordinatePair2.height;
	}
	
	tilesByHeight.sort(sortByHeight);
	
	// Generate waterDrainMap
	var waterDrainMap = deepcopy(heightmap); // This is just the amount of water draining through this field and includes no information about the direction it flows!
	for (var x = 0; x < heightmap.length; x++)
		for (var y = 0; y < heightmap[x].length; y++)
			waterDrainMap[x][y] = 0;
	
	// Simulate water drain (ignoring water height)
	var fields = [{"x" : 1, "y" : 0}, {"x" : 1, "y" : 1}, {"x" : 0, "y" : 1}, {"x" : -1, "y" : 1}, {"x" : -1, "y" : 0}, {"x" : -1, "y" : -1}, {"x" : 0, "y" : -1}, {"x" : 1, "y" : -1}]; // 8 way map
	for (var tile = 0; tile < tilesByHeight.length; tile++)
	{
		var actualX = tilesByHeight[tile].x;
		var actualY = tilesByHeight[tile].y;
		var actualWater = (1 - seepage) * waterMap[actualX][actualY];
		
		// Get total height difference to lower fields
		var sumHeightDiff = 0;
		for (var i = 0; i < fields.length; i++)
		{
			var compareX = actualX + fields[i].x;
			var compareY = actualY + fields[i].y;
			if (compareX >= 0 && compareX < heightmap.length && compareY >= 0 && compareY < heightmap[0].length)
			{
				var actualHeight = heightmap[actualX][actualY];
				var compareHeight = heightmap[compareX][compareY];
				if (compareHeight < actualHeight)
					sumHeightDiff += actualHeight - compareHeight;
			}
		}
		
		// Distribute water (drain) according to hight difference (assuming linearity)
		if (sumHeightDiff)
		{
			for (var i = 0; i < fields.length; i++)
			{
				var compareX = actualX + fields[i].x;
				var compareY = actualY + fields[i].y;
				if (compareX >= 0 && compareX < heightmap.length && compareY >= 0 && compareY < heightmap[0].length)
				{
					var actualHeight = heightmap[actualX][actualY];
					var compareHeight = heightmap[compareX][compareY];
					if (compareHeight < actualHeight)
					{
						var waterDrain = actualWater * (actualHeight - compareHeight) / sumHeightDiff;
						// waterMap[actualX][actualY] -= waterDrain; // Unneeded
						waterMap[compareX][compareY] += waterDrain;
						waterDrainMap[compareX][compareY] += waterDrain;
						waterDrainMap[actualX][actualY] += waterDrain; // Is this wanted?
					}
				}
			}
		}
	}
	
	return waterDrainMap;
}

function getWaterErodedHeightmap(heightmap, strength)
{
	strength = (strength || 1);
	var newHeightmap = deepcopy(heightmap);
	var waterAmountMap = getWaterDrainMap(heightmap, 0);
	for (var x = 0; x < heightmap.length - 1; x++)
	{
		for (var y = 0; y < heightmap[x].length - 1; y++)
		{
			// X direction
			var carriedMaterialX = strength * (heightmap[x][y] - heightmap[x+1][y]);
			if (carriedMaterialX > 0)
			{
				carriedMaterialX *= waterAmountMap[x][y]/* * (heightmap[x][y] - heightmap[x+1][y])*/;
				// newHeightmap[x][y] -= carriedMaterialX;
				// newHeightmap[x+1][y] += carriedMaterialX;
			}
			else
			{
				carriedMaterialX *= waterAmountMap[x+1][y]/* * (heightmap[x][y] - heightmap[x+1][y])*/;
				// newHeightmap[x][y] += carriedMaterialX;
				// newHeightmap[x+1][y] -= carriedMaterialX;
			}
			newHeightmap[x][y] -= carriedMaterialX;
			newHeightmap[x+1][y] += carriedMaterialX;
			// Y direction
			var carriedMaterialY = strength * (heightmap[x][y] - heightmap[x][y+1]);
			if (carriedMaterialY > 0)
			{
				carriedMaterialY *= waterAmountMap[x][y]/* * (heightmap[x][y] - heightmap[x][y+1])*/;
				// newHeightmap[x][y] -= carriedMaterialY;
				// newHeightmap[x][y+1] += carriedMaterialY;
			}
			else
			{
				carriedMaterialY *= waterAmountMap[x][y+1]/* * (heightmap[x][y] - heightmap[x][y+1])*/;
				// newHeightmap[x][y] += carriedMaterialY;
				// newHeightmap[x][y+1] -= carriedMaterialY;
			}
			newHeightmap[x][y] -= carriedMaterialY;
			newHeightmap[x][y+1] += carriedMaterialY;
		}
	}
	// Top right field missing
	// X direction
	var carriedMaterialX = strength * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-2][heightmap.length-1]);
	if (carriedMaterialX > 0)
	{
		carriedMaterialX *= waterAmountMap[heightmap.length-1][heightmap.length-1]/* * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-2][heightmap.length-1])*/;
		// newHeightmap[heightmap.length-1][heightmap.length-1] -= carriedMaterialX;
		// newHeightmap[heightmap.length-2][heightmap.length-1] += carriedMaterialX;
	}
	else
	{
		carriedMaterialX *= waterAmountMap[heightmap.length-2][heightmap.length-1]/* * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-2][heightmap.length-1])*/;
		// newHeightmap[heightmap.length-1][heightmap.length-1] += carriedMaterialX;
		// newHeightmap[heightmap.length-2][heightmap.length-1] -= carriedMaterialX;
	}
	newHeightmap[heightmap.length-1][heightmap.length-1] -= carriedMaterialX;
	newHeightmap[heightmap.length-2][heightmap.length-1] += carriedMaterialX;
	// // Y direction
	var carriedMaterialY = strength * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-1][heightmap.length-2]);
	if (carriedMaterialY > 0)
	{
		carriedMaterialY *= waterAmountMap[heightmap.length-1][heightmap.length-1]/* * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-1][heightmap.length-2])*/;
		// newHeightmap[heightmap.length-1][heightmap.length-1] -= carriedMaterialY;
		// newHeightmap[heightmap.length-1][heightmap.length-2] += carriedMaterialY;
	}
	else
	{
		carriedMaterialY *= waterAmountMap[heightmap.length-1][heightmap.length-2]/* * (heightmap[heightmap.length-1][heightmap.length-1] - heightmap[heightmap.length-1][heightmap.length-2])*/;
		// newHeightmap[heightmap.length-1][heightmap.length-1] += carriedMaterialY;
		// newHeightmap[heightmap.length-1][heightmap.length-2] -= carriedMaterialY;
	}
	newHeightmap[heightmap.length-1][heightmap.length-1] -= carriedMaterialY;
	newHeightmap[heightmap.length-1][heightmap.length-2] += carriedMaterialY;
	
	return newHeightmap;
}

//////////
// Actually set initial heightmap and manipulate it
//////////

// Set target min and max height depending on map size to make average stepness the same on all map sizes
var heightRange = {"min": MIN_HEIGHT * mapSize / 512, "max": MAX_HEIGHT * mapSize / 512};
// Override hightRange to max range
// var heightRange = {"min": MIN_HEIGHT, "max": MAX_HEIGHT};

// Set average water coverage
var averageWaterCoverage = 2/5; // NOTE: Since erosion is not pedictable actual water coverage might vary much with the same values
var waterHeight = -MIN_HEIGHT + heightRange.min + averageWaterCoverage * (heightRange.max - heightRange.min);
var waterHeightAdjusted = waterHeight + MIN_HEIGHT;
setWaterHeight(waterHeight);


// ---- Generate initial reliefmap ----

// Random heightmap
var myReliefmap = getRandomReliefmap(heightRange.min, heightRange.max);

// Flat ground at min height
// var myReliefmap = getRandomReliefmap(heightRange.min, heightRange.min);
// for (var x = 0; x < mapSize + 1; x++)
	// for (var y = 0; y < mapSize + 1; y++)
		// myReliefmap[x][y] = heightRange.min;
// Add small spike (right)
// myReliefmap[mapSize/2 + mapSize/4][mapSize/2] = heightRange.max; 

// Add brought spike (top)
// myReliefmap[mapSize/2][mapSize/2 + mapSize/4] = heightRange.min + (heightRange.max - heightRange.min) / 4;
// myReliefmap[mapSize/2][mapSize/2 + mapSize/4 + 1] = heightRange.min + (heightRange.max - heightRange.min) / 4;
// myReliefmap[mapSize/2 + 1][mapSize/2 + mapSize/4] = heightRange.min + (heightRange.max - heightRange.min) / 4;
// myReliefmap[mapSize/2 + 1][mapSize/2 + mapSize/4 + 1] = heightRange.min + (heightRange.max - heightRange.min) / 4;

// Smooth central hill
// for (var x = 0; x < mapSize + 1; x++)
// {
	// for (var y = 0; y < mapSize + 1; y++)
	// {
		// var radius = Math.pow((x - mapSize/2)*(x - mapSize/2) + (y - mapSize/2)*(y - mapSize/2), 1/2);
		// myReliefmap[x][y] = heightRange.max * cos(radius/(mapSize/2) * PI);
	// }
// }

// Central cylindric hill
// for (var x = 0; x < mapSize + 1; x++)
// {
	// for (var y = 0; y < mapSize + 1; y++)
	// {
		// var radius = Math.pow((x - mapSize/2)*(x - mapSize/2) + (y - mapSize/2)*(y - mapSize/2), 1/2);
		// if (radius > mapSize/4)
		// {
			// myReliefmap[x][y] = heightRange.min;
		// }
		// else
		// {
			// myReliefmap[x][y] = heightRange.max;
		// }
	// }
// }

// Side hills
// for (var x = 0; x < mapSize + 1; x++)
	// myReliefmap[x][0] = heightRange.max;
// for (var y = 0; y < mapSize + 1; y++)
	// myReliefmap[0][y] = heightRange.max;

// Ramps with slight irregularities
for (var x = 0; x < mapSize + 1; x++)
{
	for (var y = 0; y < mapSize + 1; y++)
	{
		// Ramp x
		myReliefmap[x][y] = x + randFloat(- mapSize / 10, mapSize / 10);
		// Ramp y
		// myReliefmap[x][y] = y + randFloat(- mapSize / 10, mapSize / 10);
		// Ramp xy
		// myReliefmap[x][y] = x + y + randFloat(- 2 * mapSize / 10, 2 * mapSize / 10);
		// Pipe
		// myReliefmap[x][y] = Math.pow(x*x + y*y, 1/2) + randFloat(- Math.pow(2, 1/2) * mapSize / 10, Math.pow(2, 1/2) * mapSize / 10);
		// Hill
		// myReliefmap[x][y] = x * y + randFloat(- mapSize * mapSize / 10, mapSize * mapSize / 10);
	}
}


// ---- Manipulate the heightmap ----
for (var i = 0; i < 200/*2*(32 + mapSize/8)*/; i++) // Cycles depend on used getHeightErrosionedReliefmap.map and mapSize so sould be put inside getHeightErrosionedReliefmap
{
	// log("Loop " + (i + 1));
	// if (i % 1 == 0)
	// {
		// for (var xy = 0; xy < mapSize + 1; xy++)
		// {
			// myReliefmap[xy][0] = heightRange.min;
			// myReliefmap[xy][mapSize] = heightRange.min;
			// myReliefmap[0][xy] = heightRange.min;
			// myReliefmap[mapSize][xy] = heightRange.min;
		// }
	// }
	// myReliefmap[0][0] = heightRange.min;
	// myReliefmap[mapSize][0] = heightRange.min;
	// myReliefmap[0][mapSize] = heightRange.min;
	// myReliefmap[mapSize][mapSize] = heightRange.min;
	
	// myReliefmap = getHeightErrosionedReliefmap(myReliefmap, 1);
	
	// myReliefmap = getAdditiveWindErrosionedReliefmap(myReliefmap, 1.5); // Strength = 1.6 makes both sides about equaly step
	// myReliefmap = getSubstractiveWindErrosionedReliefmap(myReliefmap, 1.5); // Strength = 1.6 makes both sides about equaly step
	
	// myReliefmap = getErosionHeightmapDVA(myReliefmap, 1);
	// myReliefmap = getFloatingWaterErosionHeightmapDVA(myReliefmap, 1);
	
	// myReliefmap = getDecayErosionedHeightmapDVA2d(myReliefmap, undefined, false);
	// myReliefmap = getWindErosionedHeightmapDVA2d(myReliefmap, {"x": 1, "y": 0}, 1, true); // Not working as it should
	// myReliefmap = getFloatingWaterErosionedHeightmapDVA2d(myReliefmap, 1); // Not working as it should
	
	// myReliefmap = getRainErosionedHeightmapDVA2D(myReliefmap, 1/*rainfall*/, 10/*cycles*/, 1/*strength*/);
	
	// getWaterDrainMap (acaled to me max 1) erosion ("seepage" does not seem to have any impact)
	// myReliefmap = getDifferenceOfFields2D(myReliefmap, getDiv2d(getProductOfFields2D(getRescaledField2D(getWaterDrainMap(myReliefmap, 0), 1), getGrad2d(myReliefmap, false))));
	// Unacaled getWaterDrainMap erosion (fails with to low "seepage" values)
	// myReliefmap = getDifferenceOfFields2D(myReliefmap, getDiv2d(getProductOfFields2D(getWaterDrainMap(myReliefmap, 0.5), getGrad2d(myReliefmap, false))));
	// "Square" getWaterDrainMap (acaled to me max 1) erosion
	// myReliefmap = getDifferenceOfFields2D(myReliefmap, getDiv2d(getProductOfFields2D(getRescaledField2D(getWaterDrainMap(myReliefmap, 0, getWaterDrainMap(myReliefmap, 0)), 1), getGrad2d(myReliefmap, false))));
	// getWaterDrainMap (acaled to me max 1) applied to "sqare" erosion
	
	myReliefmap = getWaterErodedHeightmap(myReliefmap, 0.25/*0.0685*/);
	
	// myReliefmap = getRescaledReliefmap(myReliefmap, heightRange.min, heightRange.max);
}

// THIS IS FUCKING WORKING ...somehow (Resonance)
// for (var i = 0; i < 50; i++)
// {
	// myReliefmap = getWaterErodedHeightmap(myReliefmap, 0.265/*0.0685*/);
	// myReliefmap = getHeightErrosionedReliefmap(myReliefmap, 0.1);
// }

// for (var i = 0; i < 10; i++)
	// myReliefmap = getDifferenceOfFields2D(myReliefmap, getDiv2d(getProductOfFields2D(getWaterDrainMap(myReliefmap, 0.6), getGrad2d(myReliefmap, false))));
	// myReliefmap = getDifferenceOfFields2D(myReliefmap, getDiv2d(getProductOfFields2D(getWaterDrainMap(myReliefmap, 0.5, getWaterDrainMap(myReliefmap, 0.5)), getGrad2d(myReliefmap, false))));

// // TEST: This mainly digs deep holes...
// for (var i = 0; i < 20; i++)
// {
	// var waterDrainMap = getRescaledField2D(getWaterDrainMap(myReliefmap, 0), 1 / 4);
	// var slopeField = getGrad2d(myReliefmap, false);
	
	// // Get slope vector field centered to tiles (don't try to use div on that!!!)
	// var slopeFieldCentered = [];
	// for (var x = 0; x < myReliefmap.length; x++)
	// {
		// slopeFieldCentered.push([]);
		// for (var y = 0; y < myReliefmap[0].length; y++)
		// {
			// slopeFieldCentered[x].push({"x" : slopeField[x][y].x, "y" : slopeField[x][y].y});
			// if (x > 0)
			// {
				// slopeFieldCentered[x][y].x += slopeField[x-1][y].x;
				// slopeFieldCentered[x][y].y += slopeField[x-1][y].y;
				// if (y > 0)
				// {
					// slopeFieldCentered[x][y].x += slopeField[x-1][y-1].x;
					// slopeFieldCentered[x][y].y += slopeField[x-1][y-1].y;
				// }
			// }
			// else if (y > 0)
			// {
				// slopeFieldCentered[x][y].x += slopeField[x][y-1].x;
				// slopeFieldCentered[x][y].y += slopeField[x][y-1].y;
			// }
		// }
	// }
	
	// var hightChangeMap = getDiv2d(slopeField, false); // Default decay erosion driven change of the hightmap
	// var finalHightChangeMap = waterDrainMap;
	// for (var x = 0; x < myReliefmap.length; x++)
	// {
		// for (var y = 0; y < myReliefmap[0].length; y++)
		// {
			// // Scale with hight over minimum height
			// // finalHightChangeMap[x][y] *= (myReliefmap[x][y] - heightRange.min) / (heightRange.max - heightRange.min);
			
			// // Scale with decay erosion intensity (inverted since it's inverted a second time when applied)
			// // finalHightChangeMap[x][y] *= -hightChangeMap[x][y];
			
			// // Scale with slope (tile centered because div(heightmap) is shifted by 1/2 tile towards positive x and y)
			// // finalHightChangeMap[x][y] *= Math.pow(slopeFieldCentered[x][y].x * slopeFieldCentered[x][y].x + slopeFieldCentered[x][y].y * slopeFieldCentered[x][y].y, 1/2);
			
			// // Only apply hight change if above water level
			// // if (myReliefmap[x][y] > waterHeightAdjusted)
			
			// myReliefmap[x][y] -= finalHightChangeMap[x][y];
		// }
	// }
	// myReliefmap = getRescaledReliefmap(myReliefmap, heightRange.min, heightRange.max);
// }
// setWaterHeight(MIN_HEIGHT); // Remove water for testing

// for (var i = 0; i < 1; i++)
	// myReliefmap = getRainErosionedHeightmapDVA2D(myReliefmap, 1/*rainfall*/, 10/*cycles*/, 1/*strength*/);
// myReliefmap = getHeightErrosionedReliefmap(myReliefmap, 1);

// TEST2:
// var slopeVF = getGrad2d(myReliefmap);
// var decayMaterialCarriageSV = getDiv2d(slopeVF);
// var waterAmountSV = getWaterDrainMap(myReliefmap);

myReliefmap = getRescaledReliefmap(myReliefmap, heightRange.min, heightRange.max);
setReliefmap(myReliefmap);


//////////
// Apply terrain texture by height
//////////

var textueByHeight = [];
textueByHeight.push({"height": heightRange.min + 1/3 * (waterHeightAdjusted - heightRange.min), "texture": "ocean_rock_deep"});
textueByHeight.push({"height": heightRange.min + 2/3 * (waterHeightAdjusted - heightRange.min), "texture": "ocean_rock_b"});
textueByHeight.push({"height": heightRange.min + 3/3 * (waterHeightAdjusted - heightRange.min), "texture": "sand_wet_b"});
textueByHeight.push({"height": waterHeightAdjusted + 1/6 * (heightRange.max - waterHeightAdjusted), "texture": "beach_scrub_50_"});
textueByHeight.push({"height": waterHeightAdjusted + 2/6 * (heightRange.max - waterHeightAdjusted), "texture": "alpine_grass_a"});
textueByHeight.push({"height": waterHeightAdjusted + 3/6 * (heightRange.max - waterHeightAdjusted), "texture": "alpine_forrestfloor"});
textueByHeight.push({"height": waterHeightAdjusted + 4/6 * (heightRange.max - waterHeightAdjusted), "texture": "new_alpine_grass_dirt_a"});
textueByHeight.push({"height": waterHeightAdjusted + 5/6 * (heightRange.max - waterHeightAdjusted), "texture": "alpine_cliff_a"});
textueByHeight.push({"height": waterHeightAdjusted + 6/6 * (heightRange.max - waterHeightAdjusted), "texture": "alpine_snow_rocky"});
for(var x = 0; x < mapSize; x++)
{
	for (var y = 0; y < mapSize; y++)
	{
		var textureMinHeight = heightRange.min;
		for (var i = 0; i < textueByHeight.length; i++)
		{
			if (getHeight(x, y) >= textureMinHeight && getHeight(x, y) <= textueByHeight[i].height)
			{
				g_Map.setTexture(x, y, textueByHeight[i].texture);
				break;
			}
			else
			{
				textureMinHeight = textueByHeight[i].height;
			}
		}
	}
}

//////////
// Paint other Stuff
//////////

// Slope
// drawFieldAbsOnTiles(getGrad2d(myReliefmap, false));

// Slope multiplied with positive DivGrad values (else 0)
// var gradRel = getGrad2d(myReliefmap, false);
// var fieldToDraw = getDiv2d(gradRel, false);
// for (var x = 0; x < fieldToDraw.length; x++)
// {
	// for (var y = 0; y < fieldToDraw[x].length; y++)
	// {
		// if (fieldToDraw[x][y] < 0)
			// fieldToDraw[x][y] = fieldToDraw[x][y] * Math.pow(gradRel[x][y].x * gradRel[x][y].x + gradRel[x][y].y * gradRel[x][y].y, 1/2);
		// else
			// fieldToDraw[x][y] = 0;
	// }
// }
// drawFieldAbsOnTiles(fieldToDraw);

// Multi step water drain speed simulation
// paintTilesWithWaterDrain(myReliefmap);

// Draw getWaterDrainMap
// var waterDrainMap = getWaterDrainMap(myReliefmap, 0.5);
// for (var x = 0; x < myReliefmap.length; x++) // Scale by height
	// for (var y = 0; y < myReliefmap[x].length; y++)
		// waterDrainMap[x][y] *= (myReliefmap[x][y] - heightRange.min) / (heightRange.max - heightRange.min);
// drawFieldAbsOnTiles(waterDrainMap);


// Export map data
ExportMap();
